package com.bocloud.sms.booter.controller;

import com.bocloud.sms.entity.ApiToken;
import com.bocloud.sms.interfaces.ApiTokenService;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.GridBean;
import com.megatron.common.model.Pager;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

/**
 * api token访问限制控制器
 */
@RestController
@RequestMapping("/v1/api/tokens")
@Tag(name = "API Token管理")
public class ApiTokenController {

    @Autowired
    private ApiTokenService apiTokenService;

    @GetMapping("/check/{token}")
    @Operation(summary = "查看API Token")
    public GeneralResult<ApiToken> detail(@PathVariable(value = Common.TOKEN) String token) {
        return apiTokenService.get(token);
    }

    /**
     * 创建Token
     *
     * @param token
     * @param context
     * @return
     */
    @PostMapping
    @Operation(summary = "创建 ApiToken")
    public GeneralResult<ApiToken> create(@RequestBody ApiToken token, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return apiTokenService.create(token, context);
    }

    /**
     * 查询Api token列表
     *
     * @param pager
     * @param context
     * @return
     */
    @GetMapping
    @Operation(summary = "查看API Token列表")
    public GeneralResult<GridBean<ApiToken>> list(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return apiTokenService.list(pager, context);
    }

    /**
     * 修改Api token
     *
     * @param token
     * @param context
     * @return
     */
    @PutMapping
    @Operation(summary = "修改API Token")
    public GeneralResult<Void> modify(@RequestBody ApiToken token, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return apiTokenService.modify(token, context);
    }

    /**
     * 移除Api Token
     *
     * @param id
     * @param context
     * @return
     */
    @DeleteMapping("/{id}")
    @Operation(summary = "移除Api Token")
    public GeneralResult<Void> remove(@PathVariable(value = Common.ID) Long id, @Value(Common.REQ_CONTEXT) RequestContext context) {
        return apiTokenService.remove(id, context);
    }

}
