package com.bocloud.sms.booter.controller;

import com.bocloud.sms.service.internel.PluginInsideService;
import com.megatron.common.model.Result;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("/v1/plugins")
@Tag(name = "插件")
@Slf4j
public class PluginInsideController {

    private final PluginInsideService pluginInsideService;
    private final ThreadPoolTaskExecutor taskExecutor;

    public PluginInsideController(PluginInsideService pluginInsideService, ThreadPoolTaskExecutor taskExecutor) {
        this.pluginInsideService = pluginInsideService;
        this.taskExecutor = taskExecutor;
    }

    @PutMapping("/{pluginId}/start")
    @Operation(summary = "启动插件")
    public Result start(@PathVariable(value = "pluginId") String pluginId,
                        @RequestParam(value = "service") String service,
                        @RequestParam(value = "instance") String instance) {
        taskExecutor.execute(() -> pluginInsideService.start(service, instance, pluginId));
        return new Result(true, "启动插件任务已下发");
    }

    @PutMapping("/{pluginId}/stop")
    @Operation(summary = "停止插件")
    public Result stop(@PathVariable(value = "pluginId") String pluginId,
                       @RequestParam(value = "service") String service,
                       @RequestParam(value = "instance") String instance) {
        taskExecutor.execute(() -> pluginInsideService.stop(service, instance, pluginId));
        return new Result(true, "停止插件任务已下发");
    }

    @PutMapping("/install")
    @Operation(summary = "安装插件")
    public Result install(@RequestParam(value = "service") String service,
                          @RequestParam(value = "instance") String instance,
                          @RequestParam(value = "pluginPath") String pluginPath) {
        taskExecutor.execute(() -> pluginInsideService.install(service, instance, pluginPath));
        return new Result(true, "安装插件任务已下发");
    }

    @PutMapping("/{pluginId}/uninstall")
    @Operation(summary = "卸载插件")
    public Result uninstall(@PathVariable(value = "pluginId") String pluginId,
                            @RequestParam(value = "service") String service,
                            @RequestParam(value = "instance") String instance) {
        taskExecutor.execute(() -> pluginInsideService.uninstall(service, instance, pluginId));
        return new Result(true, "卸载插件任务已下发");
    }

    @PutMapping("/{pluginId}/upgrade")
    @Operation(summary = "升级插件")
    public Result upgrade(@PathVariable(value = "pluginId") String pluginId,
                          @RequestParam(value = "service") String service,
                          @RequestParam(value = "instance") String instance,
                          @RequestParam(value = "pluginPath") String pluginPath) {
        taskExecutor.execute(() -> pluginInsideService.upgrade(service, instance, pluginId, pluginPath));
        return new Result(true, "升级插件任务已下发");
    }

    @PutMapping("/{pluginId}/demote")
    @Operation(summary = "降级插件")
    public Result demote(@PathVariable(value = "pluginId") String pluginId,
                         @RequestParam(value = "service") String service,
                         @RequestParam(value = "instance") String instance,
                         @RequestParam(value = "pluginPath") String pluginPath) {
        taskExecutor.execute(() -> pluginInsideService.upgrade(service, instance, pluginId, pluginPath));
        return new Result(true, "降级插件任务已下发");
    }

}
