package com.bocloud.sms.service;

import com.bocloud.sms.entity.PortalPanel;
import com.bocloud.sms.interfaces.PortalPanelService;
import com.bocloud.sms.repository.PortalPanelRepository;
import com.google.common.collect.Maps;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.RequestContext;
import com.megatron.framework.lock.AutoCloseLock;
import com.megatron.framework.lock.LockFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author lyy
 */
@Service
@Slf4j
public class PortalPanelServiceImpl implements PortalPanelService {
    @Autowired
    private LockFactory lockFactory;
    @Autowired
    private PortalPanelRepository portalPanelRepository;

    @Override
    public GeneralResult list(String module, RequestContext context) {
        try {
            List<PortalPanel> portalPanels = null;
            if("COP".equals(module)){
                portalPanels = this.portalPanelRepository.listPortalPanelCop(module,context.getTarget());
                if(CollectionUtils.isEmpty(portalPanels)){
                    portalPanels = portalPanelRepository.queryDefaultByModule(module);
                    for (PortalPanel portalPanel : portalPanels) {
                        portalPanel.setId(null);
                        portalPanel.setCreatorId(context.getTarget());
                        portalPanel.setGmtCreate(new Date());
                        portalPanelRepository.save(portalPanel);
                    }
                }
            }else {
                portalPanels = this.portalPanelRepository.listPortalPanel(module);
            }
            return new GeneralResult(true, portalPanels, "查询成功");
        } catch (Exception e) {
            log.error("Query Portal Panel error:", e);
            return new GeneralResult(false, "查询失败");
        }
    }

    @Override
    @Transactional
    public GeneralResult reset(String module, RequestContext context) {
        String path = PortalPanel.class.getSimpleName() + "_" + module;
        try (AutoCloseLock lock = lockFactory.getACLock(path)) {
            Assert.isTrue(lock.acquire(10, TimeUnit.SECONDS), "请求超时");
            List<PortalPanel> portalPanels = portalPanelRepository.queryDefaultByModule(module);
//            if (portalPanels.isEmpty()) {
//                return new GeneralResult(false, "无出厂数据，无法恢复");
//            }
            for (PortalPanel portalPanel : portalPanels) {
                portalPanel.setId(null);
                portalPanel.setCreatorId(context.getTarget());
                portalPanel.setGmtCreate(new Date());
            }
            if("COP".equals(module)){
                portalPanelRepository.removeByModuleCop(module, context.getTarget());
            }else {
                portalPanelRepository.removeByModule(module, context.getTarget());
            }

            for (PortalPanel entity : portalPanels) {
                portalPanelRepository.save(entity);
            }
            return new GeneralResult(true, "恢复成功");
        }
    }

    @Override
    public GeneralResult save(List<PortalPanel> portalPanels, Long userId, Long tenantId,
                              Long projectId, String module) {
        try {
            Map<Long, PortalPanel> old = Maps.newHashMap();
            List<PortalPanel> panels = null;
            if("COP".equals(module)){
                panels = portalPanelRepository.listPortalPanelCop(module,userId);
            }else {
                panels = portalPanelRepository.listPortalPanel(module);
            }
            for (PortalPanel portalPanel : panels) {
                old.put(portalPanel.getId(), portalPanel);
            }
            for (PortalPanel portalPanel : portalPanels) {
                if (portalPanel.getId() == null) {
                    portalPanel.setCreatorId(userId);
                    portalPanel.setGmtCreate(new Date());
                    portalPanel.setDeleted(false);
                    portalPanel.setTenantId(tenantId);
                    portalPanel.setProjectId(projectId);
                    portalPanel.setModule(module);
                    // 没有id 走新增
                    portalPanelRepository.save(portalPanel);
                    continue;
                }
                if (old.containsKey(portalPanel.getId())) {
                    PortalPanel oldPanel = old.get(portalPanel.getId());
                    oldPanel.setGmtModify(new Date());
                    oldPanel.setMenderId(userId);
                    oldPanel.setConfig(portalPanel.getConfig());
                    oldPanel.setLocation(portalPanel.getLocation());
                    portalPanelRepository.update(oldPanel);
                    old.remove(portalPanel.getId());
                }
            }
            for (PortalPanel portalPanel : old.values()) {
                portalPanelRepository.remove(portalPanel.getId());
            }
        } catch (Exception e) {
            log.error("看板保存失败！", e);
            return new GeneralResult(false, "保存失败");
        }

        return new GeneralResult(true, "保存成功");
    }

}
