package com.bocloud.sms.service.monitor;

import com.alibaba.fastjson.JSON;
import com.megatron.common.http.HttpClient;
import com.megatron.common.model.Result;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

/**
 * @author 艾乐
 * @date 2019/11/25 15:09
 */
@Service
@Slf4j
public class PrometheusMonitor {

    private final String ERROR = "error";
    private final String SUCCESS = "success";
    private final String STATUS = "status";
    private final String APIRUNTIMEINFO = "/api/v1/status/runtimeinfo";

    /**
     * 判断Prometheus是否能连接上 true：已连接上
     *
     * @return
     */
    public boolean isPrometheusConn(String host) {
        if (ERROR.equals(httpClientConnection(host + APIRUNTIMEINFO))) {
            return false;
        } else {
            String result = httpClientConnection(host + APIRUNTIMEINFO);
            return SUCCESS.equals(JSON.parseObject(result).getString(STATUS));
        }
    }

    /**
     * 判断alertManager是否能连接上 true：已连接上
     *
     * @return
     */
    public boolean isAlertManagerConn(String host) {
        if (ERROR.equals(httpClientConnection(host + "/-/healthy"))) {
            return false;
        } else {
            return "OK".equals(httpClientConnection(host + "/-/healthy"));
        }
    }

    /**
     * 发送HTTP请求
     *
     * @param url
     * @return
     */
    private String httpClientConnection(String url) {
        String resultMessage = "";
        HttpClient client = new HttpClient();
        Result result;
        try {
            result = client.get(url);
        } catch (Exception e) {
            log.error("找不到对应的主机信息！" + url);
            return ERROR;
        }
        if (result.isSuccess()) {
            resultMessage = result.getMessage();
            //去掉开头结尾的字符串，防止转化json数据失败
            if (resultMessage.startsWith("[")) {
                resultMessage = resultMessage.trim();
                resultMessage = resultMessage.substring(1, resultMessage.length() - 1);
            }
        } else {
            log.info(url + "：连接出现错误");
            resultMessage = ERROR;
        }
        return resultMessage;
    }
}
