package com.bocloud.sms.repository;

import com.bocloud.sms.entity.AccountSecurity;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.List;
import java.util.Map;

/**
 * 账号随机数访问控制类
 *
 * @author tyl
 * @since 2020/11/16
 */
@Repository
public class AccountSecurityRepository extends BasicGenericDao<AccountSecurity, Long> {

    public AccountSecurityRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public boolean delete(Long target, RequestContext.Catalog catalog) {
        String sql = "delete from account_security where target_id = :target and category = :catalog";
        Map<String, Object> params = MapTools.simpleMap("target", target);
        params.put("catalog", catalog.name());
        return this.execute(sql, params) > 0;
    }

    public AccountSecurity getByTarget(Long target, RequestContext.Catalog catalog) {
        String sql = "select * from account_security where category = :catalog and target_id = :target";
        Map<String, Object> params = MapTools.simpleMap("catalog", catalog.name());
        params.put("target", target);
        return get(sql, params);
    }

    public AccountSecurity getByApiKey(String apiKey, RequestContext.Catalog catalog) {
        String sql = "select * from account_security where api_key = :apiKey";
        Map<String, Object> params = MapTools.simpleMap("apiKey", apiKey);
        if (null != catalog) {
            sql += " category = :catalog";
            params.put("catalog", catalog.name());
        }
        return get(sql, params);
    }

    private AccountSecurity get(String select, Map<String, Object> params) {
        List<AccountSecurity> list = this.list(AccountSecurity.class, select, params);
        Assert.isTrue(list.size() <= 1, "more than one account security in table");
        return list.isEmpty() ? null : list.get(0);
    }
}
