package com.bocloud.sms.repository;

import com.bocloud.sms.entity.Business;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.database.utils.QueryBuilder;
import com.megatron.framework.core.CurrentService;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @time 2020/6/2 周二 16:16
 */
@Repository("businessRepository")
public class BusinessRepository extends JdbcGenericDao<Business, Long> {

    public BusinessRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public List<SimpleBean> list(List<Param> params, Map<String, String> sorterMap) throws Exception {
        String sql = "select a.id,a.name from business a where is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorterMap), "a");
        Map<String, Object> paramsMap = getQueryBuilder().getParam(params);
        return list(Business.class, sql, paramsMap).stream()
                .map(business -> new SimpleBean(business.getId(), business.getName())).collect(Collectors.toList());
    }

    public List<Business> list(Integer page, Integer rows, List<Param> params, Map<String, String> sorterMap)
            throws Exception {
        String sql = "select a.* from business a where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorterMap), "a");
        Map<String, Object> paramsMap = getQueryBuilder().getParam(params);
        return this.list(Business.class, sql, paramsMap);
    }

    public int count(List<Param> params) throws Exception {
        String sql = "select count(1) from business where is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        Map<String, Object> param = getQueryBuilder().getParam(params);
        return this.countQuery(sql, param).intValue();
    }

    public boolean exist(String name) throws Exception {
        String sql = "select a.* from business a where a.is_deleted = 0 and a.name = :name";
        Map<String, Object> paramsMap = MapTools.simpleMap("name", name);
        List<Business> businessList = this.list(Business.class, sql, paramsMap);
        return !businessList.isEmpty();
    }

    public boolean remove(Long id, Long menderId) throws Exception {
        String sql =
                "update business set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    public boolean batchRemove(List<Long> ids, Long menderId) throws Exception {
        String sql =
                "update business set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id in (:ids)";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 根据编码查询
     *
     * @param code
     * @return
     * @throws Exception
     */
    public Business queryByCode(String code) throws Exception {
        String sql = "select a.* from business a where a.is_deleted = 0 and a.code = :code";
        Map<String, Object> params = MapTools.simpleMap("code", code);
        List<Business> businessList = this.list(Business.class, sql, params);
        Assert.isTrue(businessList.size() <= 1, "more than one business in table");
        return businessList.isEmpty() ? null : businessList.get(0);
    }

    public Business query(Long id) throws Exception {
        String sql = "select a.* from business a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<Business> businessList = this.list(Business.class, sql, params);
        Assert.isTrue(businessList.size() <= 1, "more than one business in table");
        return businessList.isEmpty() ? null : businessList.get(0);
    }

    /**
     * 根据租户id和业务id查询
     *
     * @param tenantId
     * @param businessId
     * @return
     * @throws Exception
     */
    public List<Business> listTenantBusiness(Long tenantId, Long businessId) {
        String sql = "select * from business where tenant_id = :tenantId and business_id = :businessId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        params.put("businessId", businessId);
        return super.list(Business.class, sql, params);
    }

    /**
     * 根据租户ID查询租户业务信息
     *
     * @param tenantId
     * @return
     * @throws Exception
     */
    public List<Business> listBusiness(Long tenantId) {
        String sql = "select * from business where tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return super.list(Business.class, sql, params);
    }
}

