package com.bocloud.sms.repository;

import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.database.utils.QueryBuilder;
import com.bocloud.sms.entity.LoginRecord;
import com.megatron.common.exception.DataBaseException;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * 用户登录记录
 *
 * @author zhangyf
 * @since 2022-03-02
 */
@Repository
public class LoginRecordRepository extends BasicGenericDao<LoginRecord, Long> {

    @Autowired
    public LoginRecordRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 查询用户登录信息
     *
     * @param pager 分页查询对象
     * @return 登录记录列表
     */
    public List<LoginRecord> list(Pager pager) {
        String sql = "select a.* from login_record a where a.category = :category ";
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        paramMap.put("category", RequestContext.Catalog.User.name());
        sql = getQueryBuilder().buildRaw(sql, pager, "a");
        return super.list(LoginRecord.class, sql, paramMap);
    }

    /**
     * 用户登录信息总数
     *
     * @param params
     * @return
     * @throws DataBaseException
     */
    public int loginTimes(List<Param> params) throws DataBaseException {
        String sql = "select count(1) from login_record where category = :category ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "");
        Map<String, Object> paramMap = MapTools.simpleMap("category", RequestContext.Catalog.User.name());
        return Integer.parseInt(super.countQuery(sql, paramMap).toString());
    }

    /**
     * 查询登录信息
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @return
     */
    public List<LoginRecord> listLoginRecord(int page, int rows, List<Param> params, Map<String, String> sorter, RequestContext.Catalog category) {
        String sql = "select * from login_record a where a.category = :category";
        Map<String, Object> paramsMap = getQueryBuilder().getParam(params);
        paramsMap.put("category", category.name());
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        return super.list(LoginRecord.class, sql, paramsMap);
    }

}
