package com.bocloud.sms.repository;

import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.bocloud.sms.entity.TenantPool;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;


/**
 * 租户资源池关系
 *
 * @author hya
 * @since 2022-03-02
 */
@Repository
public class TenantPoolRepository extends BasicGenericDao<TenantPool, Long> {

    @Autowired
    public TenantPoolRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 根据业务ID删除业务资源池授权信息
     *
     * @param tenantId
     * @return
     */
    public boolean deleteTenantPool(Long tenantId) {
        String sql = "delete from tenant_pool where tenant_id = :tenantId ";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return super.execute(sql, params) > 0;
    }

    /**
     * 根据租户ID查询资源池绑定信息
     *
     * @param tenantId
     * @return
     * @throws Exception
     */
    public List<TenantPool> listTenantPool(Long tenantId) {
        String sql = "select * from tenant_pool where tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("tenantId", tenantId);
        return super.list(TenantPool.class, sql, params);
    }

    /**
     * 根据资源池id和租户id查询
     *
     * @param tenantId
     * @param poolGroupId
     * @return
     */
    public List<TenantPool> listGroupPool(Long tenantId, Long poolGroupId) {
        String sql = "select * from tenant_pool where pool_group_id = :poolGroupId and tenant_id = :tenantId";
        Map<String, Object> params = MapTools.simpleMap("poolGroupId", poolGroupId);
        params.put("tenantId", tenantId);
        return super.list(TenantPool.class, sql, params);
    }

    /**
     * 根据 poolGroupId 删除租户资源池关系
     *
     * @param poolGroupId
     * @return
     */
    public boolean deletePoolByGroupId(Long poolGroupId) {
        String sql = "delete from tenant_pool where pool_group_id = :poolGroupId";
        Map<String, Object> params = MapTools.simpleMap("poolGroupId", poolGroupId);
        return super.execute(sql, params) > 0;
    }

    /**
     * 根据 vendorId 检查资源池
     *
     * @param vendorId
     * @return
     */
    public boolean checkPoolGroup(Long vendorId) {
        String sql = "select a.* from tenant_pool a left join pool_group pg on a.pool_group_id = pg.id where pg.vendor_id = :vendorId";
        Map<String, Object> params = MapTools.simpleMap("vendorId", vendorId);
        return super.list(TenantPool.class, sql, params).size() == 0;
    }
}
